package Message::2822;

use 5.010;
use strict;
use warnings;
use Hash::Util qw(lock_keys);
use IO::File;
use if $ENV{DEBUG} => "Smart::Comments";

my %DATA = (
    fn       => {},
    fh       => {},
    header   => {},
    body_pos => {},
);

lock_keys %DATA;

sub new {
    my $class = ref $_[0] ? ref shift : shift;
    my $self = bless \(my $x) => $class;
    my %arg = @_;

    $DATA{fn}{$self} = $arg{file};
    $DATA{fh}{$self} = my $fh = IO::File->new($DATA{fn}{$self})
      or die "Can't open $DATA{fn}{$self}: $!\n";

    local $/ = "";
    chomp($DATA{header}{$self} = <$fh>);
    $DATA{body_pos}{$self} = tell($fh);

    return $self;
}

sub header_fields {
    my ($self, $pattern) = @_;
    $pattern //= qr/\S+/;
    return grep /$pattern/i => $DATA{header}{$self} =~ /^(\S+?[: ])/mg;
}

sub header_lines {
    my $self = shift;
    my $field = shift // qr/.*/;
    my @r =
      grep /$field/i => $DATA{header}{$self} =~
      /(^\S+?[: ]\s*.*?\n)(?=^\S|\Z)/imsg;
    return @r if wantarray;

    foreach (@r) { s/\s*?\n\s+/ /mg; }
    return join "" => @r;
}

sub remove_header_lines {
    my $self = shift;
    my $pattern = shift // die "Need a pattern!";
    $DATA{header}{$self} =~ s/^$pattern.*?(?=^\S|\Z)//imsg;
}

sub add_header_line {
    my $self = shift;
    my $_ = shift;
    $_ .= "\n" unless /\n$/;
    $DATA{header}{$self} .= $_;
}

sub header_contents {
    my $self = shift;
    my $field = shift // qr/.*/;

    my @r = map { (split /[: ]/, $_, 2)[1] } $self->header_lines($field);
    return @r if wantarray;

    foreach (@r) { s/\s*?\n\s+/ /mg; }
    return join "" => @r;
}

sub orig_header {
    my $self = shift;
    my $fh   = $DATA{fh}{$self};
    seek($fh, 0, 0);
    local $/ = "";
    return <$fh>;
}

sub orig_body {
    my $self = shift;
    my $fh   = $DATA{fh}{$self};
    seek($fh, $DATA{body_pos}{$self}, 0);
    local $/ = undef;
    return <$fh>;
}

sub DESTROY {
    my $self = shift;
    foreach (keys %DATA) {
        delete $DATA{$_}{$self};
    }
}

1;

__END__

=head1 NAME

  Message::2822 - simple parser for 2822 messages

=head1 SYNOPSIS

  use Message::2822;

  my $msg = Message::2822->new(file => "mail.eml");

  my @received = $msg->header_content("received:");
  my @from_ = $msg->header_content("from ");
  my @from = $msg->header_content("from:");

=head1 DESCRIPTION

This is a simple parser for RFC2822 messages. I'm sure, such parsers
exists already about one million times. Here is yet another one.

=head1 METHODS

B<Note:> Whenever we mention a I<pattern>, you may pass the pattern 
as "qr/pattern/", giving you any freedom in choosing modifiers and so on, or
you may pass the pattern simply as a string "pattern", this well then
silently converted to "qr/pattern/i".

=over

=item B<new>(file => I<file>)

The construcor. The file is opened r/o and read. The file will not be
closed until the object disappears.

=item @list = B<header_fields>([I<pattern>])

Return a list of existing header fields, matching the pattern. 
See B<header_contents()> for information about the
returned format. (default pattern: /.*/)


=item @list = B<header_lines>([I<pattern>])

Returns the header line matching the I<pattern>. See B<header_contents()>
about the returned format. (default pattern: /.*/)

=item @list = B<header_contents>([I<pattern>])

Returns the contents of the header lines matching the pattern. (default
pattern: /.*/)

In list context a list of B<unmodified> header lines is returned.
In scalar context the header lines are de-wrapped and then returned,
delimited by single linebreak.

The I<field> has to include the colon ":" or space " ",
since it is considered a part of the field name. I<field> is case
insensitive!

=item $scalar = $B<orig_header>()

Returns the original header as it is.

=item $body = B<orig_body>()

Returns the original body as it is.

=back

=head1 AUTHOR 

Heiko Schlittermann L<hs@schlittermann.de>
